clear all
clc
set(0,'defaultfigurecolor',[1 1 1])

%% INFO

% Compute peak, long-time, and residual shear stress for flow curve
% plot, measured with new friction tester (nFT)

% Version:
%   v1: 16-11-2023

% Matlab version: R2023a 


%% initialisation
mat_type = 1; % 1: C/PEEK, 2: C/LM-PAEK
ss_first = 4.5; % mm/min, threshold, steady state evaluated at first part of curve (not at end)
rel_time = 28.5; % time after sliding stopped, for relaxation curve

% store values in loop
V_mean_vec = []; % mean sliding rate, mm/s
tau_peak_vec = []; % peak shear stress, kPa
tau_mean_vec = []; % long-time shear stress, kPa
d_tau_end_vec = []; % residual or yield stress, kPa

% clear figures
figure(1), clf(1)
figure(2), clf(2)
figure(3), clf(3)

% determine measurement range
if mat_type == 1 % C/PEEK
    serie = 103; % measurement series
    range_meas = [1 2 3 4 5 6 7 8 9 13 14 15 16 17 18 19 20 21 22 23]; % measurements in serie
    cd ../FT_C-PEEK_15kPa_385degC
elseif mat_type == 2 % C/LM-PAEK
    serie = 104; % measurement series  
    range_meas = [1 2 3 4 5 6 7 8 9 10 11 12 14 15 16 17 18 19 20 21 22 40 41]; % measurements in serie
    cd ../FT_C-LMPAEK_15kPa_365degC
end

%% Go through measurements
    
for meas_i = 1:length(range_meas) % go through selected measurements

    meas = range_meas(meas_i);

    %% load data

    % find file and get data
    file_find = dir(['*',num2str(serie),'-',num2str(meas),'.txt']);
    file_name = file_find.name; 
    data = readmatrix(file_name);
    
    %% interpret data
    
    t = data(:,1) - data(1,1); % time [s]
    d = data(:,2) - data(1,2); % displacement [mm]
    f_pull = data(:,3); % pull force [N]
    
    %% compute shear stress

    A = 100*50; % contact area, mm^2      
    tau = f_pull/(2*A)*1e3; % shear stress; 2 slip interfaces, from MPa to kPa

    %% determine start and end points end mean sliding rate

    % velocity from file name
    name_u = find(file_find.name=='u');
    name_u_end = name_u + 1;
    while file_find.name(name_u_end) ~= '_'
        name_u_end = name_u_end + 1;
    end
    vel = str2double(file_find.name(name_u+1:name_u_end-1)); % velocity from file name

    % determine window for smoothening based on velocity from file name
    if file_name(end-2:end) == 'txt'  % data logged on laptop (higher sampling rate; larger window for smoothening)
        if vel <= 1
            window = 525;
        elseif vel <= 3
            window = 500;
        elseif vel <= 5
            window = 200;
        elseif vel <= 11
            window = 50;
        elseif vel <= 15
            window = 45;
        elseif vel <= 30
            window = 40;
        elseif vel <= 50
            window = 25;
        else
            window = 15;
        end
    end

    % smoothening data
    d_filt = smoothdata(d,'gaussian',window); % filter data met moving median filter

    v_vec = [0]; % velocity vector
    v_vec_1 = [0]; % velocity vector 
    a_vec = [0]; % acceleration vector
    for i = 2:length(t)-1
        dt_L = t(i) - t(i-1);
        dt_R = t(i+1) - t(i);
        v = (d_filt(i+1)-d_filt(i-1))/(dt_R+dt_L);
        a = (d_filt(i+1)-2*d_filt(i)+d_filt(i-1))/(0.5*(dt_R^2+dt_L^2)); % actually: error due to f' times difference in time step
        v_vec(i) = v;
        a_vec(i) = a;

        % first order derivative
        v_vec_1(i) = (d_filt(i+1)-d_filt(i))/(t(i+1)-t(i));
    end
    v_vec(length(t)) = v;
    v_vec_1(length(t)) = v_vec_1(length(t)-1);
    v_vec_grad = gradient(d_filt(:)) ./ gradient(t(:));
    a_vec(length(t)) = a;

    % smooth data at very low V
    if vel == 1
        a_vec = smoothdata(a_vec,'gaussian',50);
    elseif vel >= 2 && vel < 5
        a_vec = smoothdata(a_vec,'gaussian',10);
    elseif vel == 5
        a_vec = smoothdata(a_vec,'gaussian',5);
    end

    % find global start and end points
    i_glob = 1; % global end range of start point
    while d(i_glob) < 1 % deformation below 1 mm
        i_glob = i_glob + 1;
    end
    
    i_glob_9 = i_glob; % global end range of start point
    while d(i_glob_9) < 9 % deformation around 9 mm
        i_glob_9 = i_glob_9 + 1;
    end
    
    i_glob_start = i_glob; % find around i_glob also global start range for start point
    while t(i_glob_start) > t(i_glob)-(2/vel*60) && i_glob_start > 2 % move 2 mm in time earlier based on velocity in experiment
        i_glob_start = i_glob_start - 1;
    end
            
    i_glob_end = i_glob;
    while t(i_glob_end) < t(i_glob) + d(end)/vel*60 && i_glob_end < length(t)-1 % find index for time after full displacement
        i_glob_end = i_glob_end + 1;
    end
    idx_start = find(a_vec(i_glob_start:i_glob)==max(a_vec(i_glob_start:i_glob)))+i_glob_start-1;
    idx_end = find(a_vec(i_glob_9:i_glob_end)==min(a_vec(i_glob_9:i_glob_end)))+i_glob_9-1;

    % mean sliding rate
    V_mean = (d(i_glob_9)-d(i_glob))/(t(i_glob_9)-t(i_glob)); % mm/s

    %% determine relaxation curve with residual or yield stress

    % start and end indices for relaxation
    idx_f_rel_start = idx_end - 100;
    idx_f_rel_end = idx_end + 100;
    
    % second derivative shear stress w.r.t. time
    dtau_dt_sec_vec = []; 
    for i_rel = idx_f_rel_start:idx_f_rel_end
        dt = mean([t(i_rel+1)-t(i_rel),t(i_rel)-t(i_rel-1)]);
        dtau_dt_sec = (tau(i_rel+1)-2*tau(i_rel)+tau(i_rel-1))/(dt^2);
        dtau_dt_sec_vec = [dtau_dt_sec_vec, dtau_dt_sec];
    end
    
    idx_rel_start = find(dtau_dt_sec_vec==min(dtau_dt_sec_vec))+idx_f_rel_start-2; % minus 1: to obtain correct pos, extra minus 1: to start before actual sharp decrease
    
    idx_rel_end = idx_rel_start;
    while t(idx_rel_end) < t(idx_rel_start)+rel_time   % relaxation time
        idx_rel_end = idx_rel_end + 1;
    end
    tau_rel = tau(idx_rel_start:idx_rel_end);  % vector with selected range, shear stress
    t_rel = t(idx_rel_start:idx_rel_end)-t(idx_rel_start);  % relaxation time
    d_tau_end = mean(tau_rel(end-10:end));  % residual stress after ~full relaxation

    %% determine shear stress characteristics 

    % determine range for mean shear stress and peak shear stress
    if vel <= ss_first % evaluate tau_mean in first part of the curve, as a small but gradual increase is seen with time at low velocity (perhaps due to the longer test times involved)
        t_ss_first = t(idx_start) + 1/vel*60;  %start after 1 mm 
        ss_idx_start = idx_start;
        while t(ss_idx_start) < t_ss_first
            ss_idx_start = ss_idx_start + 1;
        end
        t_ss_end_first = t_ss_first + 1*60/vel;    %end after another 1 mm extra
        ss_idx_end = ss_idx_start;
        while t(ss_idx_end) < t_ss_end_first
            ss_idx_end = ss_idx_end + 1;
        end
        tau_mean = mean(tau(ss_idx_start:ss_idx_end));

        % take peak and long-time shear stress equal
        idx_peak = idx_start;
        tau_peak = tau_mean;
        
    else % peak in friction response
        ss_end = min(d(idx_rel_start),10);  % max slip length or end of measurement (which comes first)
        ss_start = ss_end - 2;  % 2 mm steady state region (~start at 8 mm)

        ss_idx_start = idx_start;
        while d(ss_idx_start) < ss_start
            ss_idx_start = ss_idx_start + 1;
        end
        ss_idx_end = ss_idx_start;
        while d(ss_idx_end) < ss_end
            ss_idx_end = ss_idx_end + 1;
        end
        tau_mean = mean(tau(ss_idx_start:ss_idx_end));

        % determine peak
        idx_peak = find(tau(idx_start:ss_idx_end)==max(tau(idx_start:ss_idx_end)))+idx_start-1;
        idx_peak = min(idx_peak); % in case multiple values at same stress level: pick first one
        tau_peak = tau(idx_peak);
    end
    
    %% store data

    V_mean_vec = [V_mean_vec,V_mean];
    tau_peak_vec = [tau_peak_vec,tau_peak];
    tau_mean_vec = [tau_mean_vec,tau_mean];
    d_tau_end_vec = [d_tau_end_vec, d_tau_end]; 
    
    %% plot data
    
    % friction response
    figure(2), hold on
    plot(d,f_pull,'-k')
    plot(d(idx_peak),f_pull(idx_peak),'xg','markersize',15);
    plot(d(ss_idx_start),f_pull(ss_idx_start),'xr','markersize',15);
    plot(d(ss_idx_end),f_pull(ss_idx_end),'xb','markersize',15);
    xlabel('Displacement [mm]')
    ylabel('Pull force [N]')
    box on
    grid on  

    % relaxation
    figure(3), hold on
    plot(t_rel,tau_rel,'-k')
    xlabel('Recovery time [s]','interpreter','latex','fontsize',14)
    ylabel('Shear stress [kPa]','interpreter','latex','fontsize',14)
    box on
    grid on
end

%% Outside the loop

cd ../Scripts
figure(2), legend('Friction data','Peak','Start steady-state','End steady-state')

% flow curve with characteristics of friction response
figure(1), hold on
plot(V_mean_vec,(tau_peak_vec-d_tau_end_vec),'k^','markersize',6,'displayname',['$\tau_{\rm{mat,p}}$'])
plot(V_mean_vec,(tau_mean_vec-d_tau_end_vec),'ko','markersize',6,'displayname',['$\tau_{\rm{mat,\infty}}$'])
legend('location','SouthEast','interpreter','latex','box','off','fontsize',14)
xlabel('Rate, $V$ [mm/s]','interpreter','Latex','fontsize',16)
ylabel('Shear stress (matrix), $\tau_{\rm{mat}}$ [kPa]','interpreter','Latex','fontsize',16)
set(gca,'XScale','log','YScale','log','fontsize',16)
box on
grid on
axis([1e-2 5 1 175])


%% save data

%save('data_points','V_mean_vec','tau_mean_vec','tau_peak_vec','d_tau_end_vec');













